@@ -666,8 +666,15 @@ translate_coords (ClutterStageX11 *stage_x11,
                   gfloat          *x_out,
                   gfloat          *y_out)
 {
-  *x_out = CLAMP (event_x, 0, stage_x11->xwin_width)  / stage_x11->scale_factor;
-  *y_out = CLAMP (event_y, 0, stage_x11->xwin_height) / stage_x11->scale_factor;
+  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
+  ClutterActor *stage = CLUTTER_ACTOR (stage_cogl->wrapper);
+  gfloat stage_width;
+  gfloat stage_height;
+
+  clutter_actor_get_size (stage, &stage_width, &stage_height);
+
+  *x_out = CLAMP (event_x, 0, stage_width)  / stage_x11->scale_factor;
+  *y_out = CLAMP (event_y, 0, stage_height) / stage_x11->scale_factor;
 }
 
 static gdouble
@@ -1148,6 +1155,14 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
 #ifdef HAVE_XINPUT_2_2
     case XI_TouchBegin:
+      {
+        XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
+        device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                      GINT_TO_POINTER (xev->deviceid));
+        if (!_clutter_input_device_get_stage (device))
+          _clutter_input_device_set_stage (device, stage);
+      }
+      /* Fall through */
     case XI_TouchEnd:
       {
         XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
